@@ -98,6 +98,8 @@ static GstAllocTrace *_gst_object_trace;
 #define DEBUG_REFCOUNT
 
 /* Object signals and args */
+/* FIXME-0.11: have a read-only parent property instead of the two signals
+ * then we get notify::parent for free */
 enum
 {
   PARENT_SET,
@@ -131,16 +133,11 @@ typedef struct _GstSignalObject GstSignalObject;
 typedef struct _GstSignalObjectClass GstSignalObjectClass;
 
 static GType gst_signal_object_get_type (void);
-static void gst_signal_object_class_init (GstSignalObjectClass * klass);
-static void gst_signal_object_init (GstSignalObject * object);
 
 #ifndef GST_DISABLE_LOADSAVE
 static guint gst_signal_object_signals[SO_LAST_SIGNAL] = { 0 };
 #endif
 
-static void gst_object_class_init (GstObjectClass * klass);
-static void gst_object_init (GTypeInstance * instance, gpointer g_class);
-
 static void gst_object_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
 static void gst_object_get_property (GObject * object, guint prop_id,
@@ -161,38 +158,12 @@ static void gst_object_real_restore_thyself (GstObject * object,
 static GObjectClass *parent_class = NULL;
 static guint gst_object_signals[LAST_SIGNAL] = { 0 };
 
-GType
-gst_object_get_type (void)
-{
-  static GType gst_object_type = 0;
-
-  if (G_UNLIKELY (gst_object_type == 0)) {
-    static const GTypeInfo object_info = {
-      sizeof (GstObjectClass),
-      NULL,
-      NULL,
-      (GClassInitFunc) gst_object_class_init,
-      NULL,
-      NULL,
-      sizeof (GstObject),
-      0,
-      gst_object_init,
-      NULL
-    };
-
-    gst_object_type =
-        g_type_register_static (G_TYPE_OBJECT, "GstObject", &object_info,
-        G_TYPE_FLAG_ABSTRACT);
-  }
-  return gst_object_type;
-}
+G_DEFINE_ABSTRACT_TYPE (GstObject, gst_object, G_TYPE_OBJECT);
 
 static void
 gst_object_class_init (GstObjectClass * klass)
 {
-  GObjectClass *gobject_class;
-
-  gobject_class = G_OBJECT_CLASS (klass);
+  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
   parent_class = g_type_class_peek_parent (klass);
 
@@ -200,8 +171,8 @@ gst_object_class_init (GstObjectClass * klass)
   _gst_object_trace = gst_alloc_trace_register (g_type_name (GST_TYPE_OBJECT));
 #endif
 
-  gobject_class->set_property = GST_DEBUG_FUNCPTR (gst_object_set_property);
-  gobject_class->get_property = GST_DEBUG_FUNCPTR (gst_object_get_property);
+  gobject_class->set_property = gst_object_set_property;
+  gobject_class->get_property = gst_object_get_property;
 
   g_object_class_install_property (gobject_class, ARG_NAME,
       g_param_spec_string ("name", "Name", "The name of the object",
@@ -270,10 +241,11 @@ gst_object_class_init (GstObjectClass * klass)
       G_TYPE_PARAM);
 
   klass->path_string_separator = "/";
-  klass->lock = g_new0 (GStaticRecMutex, 1);
+  /* FIXME 0.11: Store this directly in the class struct */
+  klass->lock = g_slice_new (GStaticRecMutex);
   g_static_rec_mutex_init (klass->lock);
 
-  klass->signal_object = g_object_new (gst_signal_object_get_type (), NULL);
+  klass->signal_object = g_object_newv (gst_signal_object_get_type (), 0, NULL);
 
   /* see the comments at gst_object_dispatch_properties_changed */
   gobject_class->dispatch_properties_changed
@@ -284,10 +256,8 @@ gst_object_class_init (GstObjectClass * klass)
 }
 
 static void
-gst_object_init (GTypeInstance * instance, gpointer g_class)
+gst_object_init (GstObject * object)
 {
-  GstObject *object = GST_OBJECT (instance);
-
   object->lock = g_mutex_new ();
   object->parent = NULL;
   object->name = NULL;
@@ -355,6 +325,39 @@ gst_object_unref (gpointer object)
   g_object_unref (object);
 }
 
+/**
+ * gst_object_ref_sink:
+ * @object: a #GstObject to sink
+ *
+ * Increase the reference count of @object, and possibly remove the floating
+ * reference, if @object has a floating reference.
+ *
+ * In other words, if the object is floating, then this call "assumes ownership"
+ * of the floating reference, converting it to a normal reference by clearing
+ * the floating flag while leaving the reference count unchanged. If the object
+ * is not floating, then this call adds a new normal reference increasing the
+ * reference count by one.
+ *
+ * MT safe. This function grabs and releases @object lock.
+ *
+ * Since: 0.10.24
+ */
+void
+gst_object_ref_sink (gpointer object)
+{
+  g_return_if_fail (GST_IS_OBJECT (object));
+
+  GST_OBJECT_LOCK (object);
+  if (G_LIKELY (GST_OBJECT_IS_FLOATING (object))) {
+    GST_CAT_LOG_OBJECT (GST_CAT_REFCOUNTING, object, "unsetting floating flag");
+    GST_OBJECT_FLAG_UNSET (object, GST_OBJECT_FLOATING);
+    GST_OBJECT_UNLOCK (object);
+  } else {
+    GST_OBJECT_UNLOCK (object);
+    gst_object_ref (object);
+  }
+}
+
 /**
  * gst_object_sink:
  * @object: a #GstObject to sink
@@ -412,10 +415,10 @@ gst_object_replace (GstObject ** oldobj, GstObject * newobj)
   g_return_if_fail (newobj == NULL || GST_IS_OBJECT (newobj));
 
 #ifdef DEBUG_REFCOUNT
-  GST_CAT_LOG (GST_CAT_REFCOUNTING, "replace %s (%d) with %s (%d)",
-      *oldobj ? GST_STR_NULL (GST_OBJECT_NAME (*oldobj)) : "(NONE)",
+  GST_CAT_LOG (GST_CAT_REFCOUNTING, "replace %p %s (%d) with %p %s (%d)",
+      *oldobj, *oldobj ? GST_STR_NULL (GST_OBJECT_NAME (*oldobj)) : "(NONE)",
       *oldobj ? G_OBJECT (*oldobj)->ref_count : 0,
-      newobj ? GST_STR_NULL (GST_OBJECT_NAME (newobj)) : "(NONE)",
+      newobj, newobj ? GST_STR_NULL (GST_OBJECT_NAME (newobj)) : "(NONE)",
       newobj ? G_OBJECT (newobj)->ref_count : 0);
 #endif
 
@@ -457,7 +460,7 @@ have_parent:
         GST_OBJECT_NAME (object), GST_OBJECT_NAME (parent));
     GST_OBJECT_UNLOCK (object);
     /* ref the object again to revive it in this error case */
-    object = gst_object_ref (object);
+    gst_object_ref (object);
     return;
   }
 }
@@ -466,7 +469,7 @@ have_parent:
 static void
 gst_object_finalize (GObject * object)
 {
-  GstObject *gstobject = GST_OBJECT (object);
+  GstObject *gstobject = GST_OBJECT_CAST (object);
 
   GST_CAT_LOG_OBJECT (GST_CAT_REFCOUNTING, object, "finalize");
 
@@ -482,7 +485,7 @@ gst_object_finalize (GObject * object)
   parent_class->finalize (object);
 }
 
-/* Changing a GObject property of a GstObject will result in "deep_notify"
+/* Changing a GObject property of a GstObject will result in "deep-notify"
  * signals being emitted by the object itself, as well as in each parent
  * object. This is so that an application can connect a listener to the
  * top-level bin to catch property-change notifications for all contained
@@ -499,17 +502,11 @@ gst_object_dispatch_properties_changed (GObject * object,
 {
   GstObject *gst_object, *parent, *old_parent;
   guint i;
-  gchar *name, *debug_name;
-  GstObjectClass *klass;
-
-  /* we fail when this is not a GstObject */
-  g_return_if_fail (GST_IS_OBJECT (object));
-
-  klass = GST_OBJECT_GET_CLASS (object);
+  gchar *name;
+  const gchar *debug_name;
 
   /* do the standard dispatching */
-  G_OBJECT_CLASS (parent_class)->dispatch_properties_changed (object, n_pspecs,
-      pspecs);
+  parent_class->dispatch_properties_changed (object, n_pspecs, pspecs);
 
   gst_object = GST_OBJECT_CAST (object);
   name = gst_object_get_name (gst_object);
@@ -523,8 +520,7 @@ gst_object_dispatch_properties_changed (GObject * object,
           "deep notification from %s (%s)", debug_name, pspecs[i]->name);
 
       g_signal_emit (parent, gst_object_signals[DEEP_NOTIFY],
-          g_quark_from_string (pspecs[i]->name), GST_OBJECT_CAST (object),
-          pspecs[i]);
+          g_quark_from_string (pspecs[i]->name), gst_object, pspecs[i]);
     }
 
     old_parent = parent;
@@ -565,7 +561,7 @@ gst_object_default_deep_notify (GObject * object, GstObject * orig,
         return;
       excluded_props++;
     }
-    g_value_init (&value, G_PARAM_SPEC_VALUE_TYPE (pspec));
+    g_value_init (&value, pspec->value_type);
     g_object_get_property (G_OBJECT (orig), pspec->name, &value);
 
     /* FIXME: handle flags */
@@ -599,7 +595,6 @@ gst_object_set_name_default (GstObject * object)
   const gchar *type_name;
   gint count;
   gchar *name, *tmp;
-  gboolean result;
   GQuark q;
 
   /* to ensure guaranteed uniqueness across threads, only one thread
@@ -624,10 +619,22 @@ gst_object_set_name_default (GstObject * object)
   name = g_ascii_strdown (tmp, -1);
   g_free (tmp);
 
-  result = gst_object_set_name (object, name);
-  g_free (name);
+  GST_OBJECT_LOCK (object);
+  if (G_UNLIKELY (object->parent != NULL))
+    goto had_parent;
+  g_free (object->name);
+  object->name = name;
 
-  return result;
+  GST_OBJECT_UNLOCK (object);
+
+  return TRUE;
+
+had_parent:
+  {
+    GST_WARNING ("parented objects can't be renamed");
+    GST_OBJECT_UNLOCK (object);
+    return FALSE;
+  }
 }
 
 /**
@@ -668,6 +675,10 @@ gst_object_set_name (GstObject * object, const gchar * name)
     GST_OBJECT_UNLOCK (object);
     result = gst_object_set_name_default (object);
   }
+  /* FIXME-0.11: this misses a g_object_notify (object, "name"); unless called
+   * from gst_object_set_property.
+   * Ideally remove such custom setters (or make it static).
+   */
   return result;
 
   /* error */
@@ -798,7 +809,7 @@ gst_object_set_parent (GstObject * object, GstObject * parent)
     gst_object_ref (object);
   }
 
-  g_signal_emit (G_OBJECT (object), gst_object_signals[PARENT_SET], 0, parent);
+  g_signal_emit (object, gst_object_signals[PARENT_SET], 0, parent);
 
   return TRUE;
 
@@ -864,8 +875,7 @@ gst_object_unparent (GstObject * object)
     object->parent = NULL;
     GST_OBJECT_UNLOCK (object);
 
-    g_signal_emit (G_OBJECT (object), gst_object_signals[PARENT_UNSET], 0,
-        parent);
+    g_signal_emit (object, gst_object_signals[PARENT_UNSET], 0, parent);
 
     gst_object_unref (object);
   } else {
@@ -888,21 +898,24 @@ gst_object_unparent (GstObject * object)
 gboolean
 gst_object_has_ancestor (GstObject * object, GstObject * ancestor)
 {
-  GstObject *parent;
-  gboolean result = FALSE;
+  GstObject *parent, *tmp;
 
-  if (object == NULL)
+  if (!ancestor || !object)
     return FALSE;
 
-  if (object == ancestor)
-    return TRUE;
+  parent = gst_object_ref (object);
+  do {
+    if (parent == ancestor) {
+      gst_object_unref (parent);
+      return TRUE;
+    }
 
-  parent = gst_object_get_parent (object);
-  result = gst_object_has_ancestor (parent, ancestor);
-  if (parent)
+    tmp = gst_object_get_parent (parent);
     gst_object_unref (parent);
+    parent = tmp;
+  } while (parent);
 
-  return result;
+  return FALSE;
 }
 
 /**
@@ -932,7 +945,7 @@ gst_object_check_uniqueness (GList * list, const gchar * name)
     GstObject *child;
     gboolean eq;
 
-    child = GST_OBJECT (list->data);
+    child = GST_OBJECT_CAST (list->data);
 
     GST_OBJECT_LOCK (child);
     eq = strcmp (GST_OBJECT_NAME (child), name) == 0;
@@ -970,8 +983,7 @@ gst_object_save_thyself (GstObject * object, xmlNodePtr parent)
   if (oclass->save_thyself)
     oclass->save_thyself (object, parent);
 
-  g_signal_emit (G_OBJECT (object), gst_object_signals[OBJECT_SAVED], 0,
-      parent);
+  g_signal_emit (object, gst_object_signals[OBJECT_SAVED], 0, parent);
 
   return parent;
 }
@@ -1013,7 +1025,7 @@ gst_object_set_property (GObject * object, guint prop_id,
 {
   GstObject *gstobject;
 
-  gstobject = GST_OBJECT (object);
+  gstobject = GST_OBJECT_CAST (object);
 
   switch (prop_id) {
     case ARG_NAME:
@@ -1031,7 +1043,7 @@ gst_object_get_property (GObject * object, guint prop_id,
 {
   GstObject *gstobject;
 
-  gstobject = GST_OBJECT (object);
+  gstobject = GST_OBJECT_CAST (object);
 
   switch (prop_id) {
     case ARG_NAME:
@@ -1065,7 +1077,7 @@ gst_object_get_path_string (GstObject * object)
   gchar *prevpath, *path;
   const gchar *typename;
   gchar *component;
-  gchar *separator;
+  const gchar *separator;
 
   /* ref object before adding to list */
   gst_object_ref (object);
@@ -1144,39 +1156,11 @@ struct _GstSignalObjectClass
 #endif
 };
 
-static GType
-gst_signal_object_get_type (void)
-{
-  static GType signal_object_type = 0;
-
-  if (G_UNLIKELY (signal_object_type == 0)) {
-    static const GTypeInfo signal_object_info = {
-      sizeof (GstSignalObjectClass),
-      NULL,
-      NULL,
-      (GClassInitFunc) gst_signal_object_class_init,
-      NULL,
-      NULL,
-      sizeof (GstSignalObject),
-      0,
-      (GInstanceInitFunc) gst_signal_object_init,
-      NULL
-    };
-
-    signal_object_type =
-        g_type_register_static (G_TYPE_OBJECT, "GstSignalObject",
-        &signal_object_info, 0);
-  }
-  return signal_object_type;
-}
+G_DEFINE_TYPE (GstSignalObject, gst_signal_object, G_TYPE_OBJECT);
 
 static void
 gst_signal_object_class_init (GstSignalObjectClass * klass)
 {
-  GObjectClass *gobject_class;
-
-  gobject_class = (GObjectClass *) klass;
-
   parent_class = g_type_class_peek_parent (klass);
 
 #ifndef GST_DISABLE_LOADSAVE
