@@ -50,7 +50,7 @@
  * <orderedlist>
  *   <listitem><para>
  *     first put some properties under control, by calling
- *     controller = g_object_control_properties(object, "prop1", "prop2",...);
+ *     controller = gst_object_control_properties (object, "prop1", "prop2",...);
  *   </para></listitem>
  *   <listitem><para>
  *     Get a #GstControlSource for the property and set it up.
@@ -135,17 +135,18 @@ gst_controlled_property_new (GObject * object, const gchar * name)
           g_object_class_find_property (G_OBJECT_GET_CLASS (object), name))) {
     GST_DEBUG ("  psec->flags : 0x%08x", pspec->flags);
 
-    /* check if this param is witable */
-    g_return_val_if_fail ((pspec->flags & G_PARAM_WRITABLE), NULL);
-    /* check if property is controlable */
-    g_return_val_if_fail ((pspec->flags & GST_PARAM_CONTROLLABLE), NULL);
-    /* check if this param is not construct-only */
-    g_return_val_if_fail (!(pspec->flags & G_PARAM_CONSTRUCT_ONLY), NULL);
+    /* check if this param is witable && controlable && !construct-only */
+    g_return_val_if_fail ((pspec->flags & (G_PARAM_WRITABLE |
+                GST_PARAM_CONTROLLABLE | G_PARAM_CONSTRUCT_ONLY)) ==
+        (G_PARAM_WRITABLE | GST_PARAM_CONTROLLABLE), NULL);
 
-    if ((prop = g_new0 (GstControlledProperty, 1))) {
+    if ((prop = g_slice_new (GstControlledProperty))) {
       prop->pspec = pspec;
       prop->name = pspec->name;
+      prop->csource = NULL;
       prop->disabled = FALSE;
+      memset (&prop->last_value, 0, sizeof (GValue));
+      g_value_init (&prop->last_value, G_PARAM_SPEC_VALUE_TYPE (prop->pspec));
     }
   } else {
     GST_WARNING ("class '%s' has no property '%s'", G_OBJECT_TYPE_NAME (object),
@@ -166,7 +167,8 @@ gst_controlled_property_free (GstControlledProperty * prop)
 {
   if (prop->csource)
     g_object_unref (prop->csource);
-  g_free (prop);
+  g_value_unset (&prop->last_value);
+  g_slice_free (GstControlledProperty, prop);
 }
 
 /*
@@ -198,6 +200,58 @@ gst_controller_find_controlled_property (GstController * self,
   return NULL;
 }
 
+/*
+ * gst_controller_add_property:
+ * @self: the controller object or %NULL if none yet exists
+ * @object: object to bind the property
+ * @name: name of projecty in @object
+ * @ref_existing: pointer to flag that tracks if we need to ref an existng
+ *   controller still
+ *
+ * Creates a new #GstControlledProperty if there is none for property @name yet.
+ * In case this is the first controlled propery, it creates the controller as
+ * well.
+ *
+ * Returns: a newly created controller object or reffed existing one with the
+ * given property bound.
+ */
+static GstController *
+gst_controller_add_property (GstController * self, GObject * object,
+    const gchar * name, gboolean * ref_existing)
+{
+  /* test if this property isn't yet controlled */
+  if (!self || !gst_controller_find_controlled_property (self, name)) {
+    GstControlledProperty *prop;
+
+    /* create GstControlledProperty and add to self->propeties List */
+    if ((prop = gst_controlled_property_new (object, name))) {
+      /* if we don't have a controller object yet, now is the time to create one */
+      if (!self) {
+        self = g_object_newv (GST_TYPE_CONTROLLER, 0, NULL);
+        self->object = g_object_ref (object);
+        /* store the controller */
+        g_object_set_qdata (object, priv_gst_controller_key, self);
+        *ref_existing = FALSE;
+      } else {
+        /* only want one single _ref(), even for multiple properties */
+        if (*ref_existing) {
+          g_object_ref (self);
+          *ref_existing = FALSE;
+          GST_INFO ("returning existing controller");
+        }
+      }
+      self->properties = g_list_prepend (self->properties, prop);
+    }
+  } else {
+    GST_WARNING ("trying to control property %s again", name);
+    if (*ref_existing) {
+      g_object_ref (self);
+      *ref_existing = FALSE;
+    }
+  }
+  return self;
+}
+
 /* methods */
 
 /**
@@ -213,7 +267,6 @@ GstController *
 gst_controller_new_valist (GObject * object, va_list var_args)
 {
   GstController *self;
-  GstControlledProperty *prop;
   gboolean ref_existing = TRUE;
   gchar *name;
 
@@ -224,34 +277,7 @@ gst_controller_new_valist (GObject * object, va_list var_args)
   self = g_object_get_qdata (object, priv_gst_controller_key);
   /* create GstControlledProperty for each property */
   while ((name = va_arg (var_args, gchar *))) {
-    /* test if this property isn't yet controlled */
-    if (!self || !(prop = gst_controller_find_controlled_property (self, name))) {
-      /* create GstControlledProperty and add to self->propeties List */
-      if ((prop = gst_controlled_property_new (object, name))) {
-        /* if we don't have a controller object yet, now is the time to create one */
-        if (!self) {
-          self = g_object_new (GST_TYPE_CONTROLLER, NULL);
-          self->object = g_object_ref (object);
-          /* store the controller */
-          g_object_set_qdata (object, priv_gst_controller_key, self);
-          ref_existing = FALSE;
-        } else {
-          /* only want one single _ref(), even for multiple properties */
-          if (ref_existing) {
-            g_object_ref (self);
-            ref_existing = FALSE;
-            GST_INFO ("returning existing controller");
-          }
-        }
-        self->properties = g_list_prepend (self->properties, prop);
-      }
-    } else {
-      GST_WARNING ("trying to control property again");
-      if (ref_existing) {
-        g_object_ref (self);
-        ref_existing = FALSE;
-      }
-    }
+    self = gst_controller_add_property (self, object, name, &ref_existing);
   }
   va_end (var_args);
 
@@ -273,7 +299,6 @@ GstController *
 gst_controller_new_list (GObject * object, GList * list)
 {
   GstController *self;
-  GstControlledProperty *prop;
   gboolean ref_existing = TRUE;
   gchar *name;
   GList *node;
@@ -286,34 +311,7 @@ gst_controller_new_list (GObject * object, GList * list)
   /* create GstControlledProperty for each property */
   for (node = list; node; node = g_list_next (node)) {
     name = (gchar *) node->data;
-    /* test if this property isn't yet controlled */
-    if (!self || !(prop = gst_controller_find_controlled_property (self, name))) {
-      /* create GstControlledProperty and add to self->propeties List */
-      if ((prop = gst_controlled_property_new (object, name))) {
-        /* if we don't have a controller object yet, now is the time to create one */
-        if (!self) {
-          self = g_object_new (GST_TYPE_CONTROLLER, NULL);
-          self->object = g_object_ref (object);
-          /* store the controller */
-          g_object_set_qdata (object, priv_gst_controller_key, self);
-          ref_existing = FALSE;
-        } else {
-          /* only want one single _ref(), even for multiple properties */
-          if (ref_existing) {
-            g_object_ref (self);
-            ref_existing = FALSE;
-            GST_INFO ("returning existing controller");
-          }
-        }
-        self->properties = g_list_prepend (self->properties, prop);
-      }
-    } else {
-      GST_WARNING ("trying to control property again");
-      if (ref_existing) {
-        g_object_ref (self);
-        ref_existing = FALSE;
-      }
-    }
+    self = gst_controller_add_property (self, object, name, &ref_existing);
   }
 
   if (self)
@@ -456,7 +454,7 @@ gst_controller_remove_properties (GstController * self, ...)
 
 void
 gst_controller_set_property_disabled (GstController * self,
-    gchar * property_name, gboolean disabled)
+    const gchar * property_name, gboolean disabled)
 {
   GstControlledProperty *prop;
 
@@ -514,8 +512,8 @@ gst_controller_set_disabled (GstController * self, gboolean disabled)
  * Since: 0.10.14
  */
 gboolean
-gst_controller_set_control_source (GstController * self, gchar * property_name,
-    GstControlSource * csource)
+gst_controller_set_control_source (GstController * self,
+    const gchar * property_name, GstControlSource * csource)
 {
   GstControlledProperty *prop;
   gboolean ret = FALSE;
@@ -554,7 +552,8 @@ gst_controller_set_control_source (GstController * self, gchar * property_name,
  * Since: 0.10.14
  */
 GstControlSource *
-gst_controller_get_control_source (GstController * self, gchar * property_name)
+gst_controller_get_control_source (GstController * self,
+    const gchar * property_name)
 {
   GstControlledProperty *prop;
   GstControlSource *ret = NULL;
@@ -583,10 +582,11 @@ gst_controller_get_control_source (GstController * self, gchar * property_name)
  * Gets the value for the given controller-handled property at the requested
  * time.
  *
- * Returns: the GValue of the property at the given time, or %NULL if the property isn't handled by the controller
+ * Returns: the GValue of the property at the given time, or %NULL if the
+ * property isn't handled by the controller
  */
 GValue *
-gst_controller_get (GstController * self, gchar * property_name,
+gst_controller_get (GstController * self, const gchar * property_name,
     GstClockTime timestamp)
 {
   GstControlledProperty *prop;
@@ -642,7 +642,9 @@ gst_controller_suggest_next_sync (GstController * self)
   g_mutex_lock (self->lock);
 
   /* TODO: Implement more logic, depending on interpolation mode
-   * and control points */
+   * and control points
+   * FIXME: we need playback direction
+   */
   ret = self->priv->last_sync + self->priv->control_rate;
 
   g_mutex_unlock (self->lock);
@@ -658,6 +660,9 @@ gst_controller_suggest_next_sync (GstController * self)
  * Sets the properties of the element, according to the controller that (maybe)
  * handles them and for the given timestamp.
  *
+ * If this function fails, it is most likely the application developers fault.
+ * Most probably the control sources are not setup correctly.
+ *
  * Returns: %TRUE if the controller values could be applied to the object
  * properties, %FALSE otherwise
  */
@@ -666,7 +671,7 @@ gst_controller_sync_values (GstController * self, GstClockTime timestamp)
 {
   GstControlledProperty *prop;
   GList *node;
-  gboolean ret = FALSE;
+  gboolean ret = TRUE, val_ret;
   GValue value = { 0, };
 
   g_return_val_if_fail (GST_IS_CONTROLLER (self), FALSE);
@@ -680,22 +685,31 @@ gst_controller_sync_values (GstController * self, GstClockTime timestamp)
   for (node = self->properties; node; node = g_list_next (node)) {
     prop = node->data;
 
-    GST_LOG ("property '%s' at ts=%" G_GUINT64_FORMAT, prop->name, timestamp);
-
     if (!prop->csource || prop->disabled)
       continue;
 
+    GST_LOG ("property '%s' at ts=%" G_GUINT64_FORMAT, prop->name, timestamp);
+
     /* we can make this faster
      * http://bugzilla.gnome.org/show_bug.cgi?id=536939
      */
     g_value_init (&value, G_PARAM_SPEC_VALUE_TYPE (prop->pspec));
-    ret = gst_control_source_get_value (prop->csource, timestamp, &value);
-    if (G_LIKELY (ret)) {
-      g_object_set_property (self->object, prop->name, &value);
+    val_ret = gst_control_source_get_value (prop->csource, timestamp, &value);
+    if (G_LIKELY (val_ret)) {
+      /* always set the value for first time, but then only if it changed
+       * this should limit g_object_notify invocations.
+       * FIXME: can we detect negative playback rates?
+       */
+      if ((timestamp < self->priv->last_sync) ||
+          gst_value_compare (&value, &prop->last_value) != GST_VALUE_EQUAL) {
+        g_object_set_property (self->object, prop->name, &value);
+        g_value_copy (&value, &prop->last_value);
+      }
     } else {
-      GST_LOG ("no control value");
+      GST_DEBUG ("no control value for param %s", prop->name);
     }
     g_value_unset (&value);
+    ret &= val_ret;
   }
   self->priv->last_sync = timestamp;
   g_object_thaw_notify (self->object);
@@ -936,9 +950,10 @@ _gst_controller_class_init (GstControllerClass * klass)
 GType
 gst_controller_get_type (void)
 {
-  static GType type = 0;
+  static volatile gsize type = 0;
 
-  if (type == 0) {
+  if (g_once_init_enter (&type)) {
+    GType _type;
     static const GTypeInfo info = {
       sizeof (GstControllerClass),
       NULL,                     /* base_init */
@@ -951,7 +966,8 @@ gst_controller_get_type (void)
       (GInstanceInitFunc) _gst_controller_init, /* instance_init */
       NULL                      /* value_table */
     };
-    type = g_type_register_static (G_TYPE_OBJECT, "GstController", &info, 0);
+    _type = g_type_register_static (G_TYPE_OBJECT, "GstController", &info, 0);
+    g_once_init_leave (&type, _type);
   }
   return type;
 }
@@ -1005,8 +1021,13 @@ gst_controlled_property_set_interpolation_mode (GstControlledProperty * self,
  * Returns: FALSE if the values couldn't be set (ex : properties not handled by controller), TRUE otherwise
  */
 #ifndef GST_REMOVE_DEPRECATED
+#ifdef GST_DISABLE_DEPRECATED
 gboolean
-gst_controller_set (GstController * self, gchar * property_name,
+gst_controller_set (GstController * self, const gchar * property_name,
+    GstClockTime timestamp, GValue * value);
+#endif
+gboolean
+gst_controller_set (GstController * self, const gchar * property_name,
     GstClockTime timestamp, GValue * value)
 {
   gboolean res = FALSE;
@@ -1049,8 +1070,13 @@ out:
  * Returns: %FALSE if the values couldn't be set (ex : properties not handled by controller), %TRUE otherwise
  */
 #ifndef GST_REMOVE_DEPRECATED
+#ifdef GST_DISABLE_DEPRECATED
 gboolean
-gst_controller_set_from_list (GstController * self, gchar * property_name,
+gst_controller_set_from_list (GstController * self, const gchar * property_name,
+    GSList * timedvalues);
+#endif
+gboolean
+gst_controller_set_from_list (GstController * self, const gchar * property_name,
     GSList * timedvalues)
 {
   gboolean res = FALSE;
@@ -1095,8 +1121,13 @@ out:
  * Returns: %FALSE if the values couldn't be unset (ex : properties not handled by controller), %TRUE otherwise
  */
 #ifndef GST_REMOVE_DEPRECATED
+#ifdef GST_DISABLE_DEPRECATED
 gboolean
-gst_controller_unset (GstController * self, gchar * property_name,
+gst_controller_unset (GstController * self, const gchar * property_name,
+    GstClockTime timestamp);
+#endif
+gboolean
+gst_controller_unset (GstController * self, const gchar * property_name,
     GstClockTime timestamp)
 {
   gboolean res = FALSE;
@@ -1138,8 +1169,11 @@ out:
  * Since: 0.10.5
  */
 #ifndef GST_REMOVE_DEPRECATED
+#ifdef GST_DISABLE_DEPRECATED
+gboolean gst_controller_unset_all (GstController * self, const gchar * property_name);
+#endif
 gboolean
-gst_controller_unset_all (GstController * self, gchar * property_name)
+gst_controller_unset_all (GstController * self, const gchar * property_name)
 {
   GstControlledProperty *prop;
 
@@ -1178,8 +1212,12 @@ out:
  * Returns: a copy of the list, or %NULL if the property isn't handled by the controller
  */
 #ifndef GST_REMOVE_DEPRECATED
+#ifdef GST_DISABLE_DEPRECATED
+const GList *gst_controller_get_all (GstController * self,
+    const gchar * property_name);
+#endif
 const GList *
-gst_controller_get_all (GstController * self, gchar * property_name)
+gst_controller_get_all (GstController * self, const gchar * property_name)
 {
   const GList *res = NULL;
   GstControlledProperty *prop;
@@ -1221,9 +1259,14 @@ out:
  * Returns: %TRUE if the property is handled by the controller, %FALSE otherwise
  */
 #ifndef GST_REMOVE_DEPRECATED
+#ifdef GST_DISABLE_DEPRECATED
 gboolean
 gst_controller_set_interpolation_mode (GstController * self,
-    gchar * property_name, GstInterpolateMode mode)
+    const gchar * property_name, GstInterpolateMode mode);
+#endif
+gboolean
+gst_controller_set_interpolation_mode (GstController * self,
+    const gchar * property_name, GstInterpolateMode mode)
 {
   gboolean res = FALSE;
   GstControlledProperty *prop;
