@@ -300,8 +300,9 @@ GST_START_TEST (test_message_state_changed_children)
   /* each object is referenced by a message;
    * base_src is blocked in the push and has an extra refcount.
    * base_sink_chain has taken a refcount on the sink, and is blocked on
-   * preroll */
-  ASSERT_OBJECT_REFCOUNT (src, "src", 2);
+   * preroll
+   * The stream-status messages holds 2 more refs to the element */
+  ASSERT_OBJECT_REFCOUNT (src, "src", 4);
   /* refcount can be 4 if the bin is still processing the async_done message of
    * the sink. */
   ASSERT_OBJECT_REFCOUNT_BETWEEN (sink, "sink", 2, 3);
@@ -352,8 +353,9 @@ GST_START_TEST (test_message_state_changed_children)
   ret = gst_element_set_state (GST_ELEMENT (pipeline), GST_STATE_READY);
   fail_unless (ret == GST_STATE_CHANGE_SUCCESS);
 
-  /* each object is referenced by two messages */
-  ASSERT_OBJECT_REFCOUNT (src, "src", 3);
+  /* each object is referenced by two messages, the source also has the
+   * stream-status message referencing it */
+  ASSERT_OBJECT_REFCOUNT (src, "src", 4);
   ASSERT_OBJECT_REFCOUNT (sink, "sink", 3);
   ASSERT_OBJECT_REFCOUNT (pipeline, "pipeline", 3);
 
@@ -969,6 +971,60 @@ GST_START_TEST (test_link_structure_change)
 
 GST_END_TEST;
 
+static GstBusSyncReply
+sync_handler_remove_sink (GstBus * bus, GstMessage * message, gpointer data)
+{
+  if (GST_MESSAGE_TYPE (message) == GST_MESSAGE_ERROR) {
+    GstElement *child;
+
+    child = gst_bin_get_by_name (GST_BIN (data), "fakesink");
+    fail_unless (child != NULL, "Could not find fakesink");
+
+    gst_bin_remove (GST_BIN (data), child);
+    gst_object_unref (child);
+  }
+  return GST_BUS_PASS;
+}
+
+GST_START_TEST (test_state_failure_remove)
+{
+  GstElement *src, *sink, *pipeline;
+  GstBus *bus;
+  GstStateChangeReturn ret;
+
+  pipeline = gst_pipeline_new (NULL);
+  fail_unless (pipeline != NULL, "Could not create pipeline");
+
+  src = gst_element_factory_make ("fakesrc", "fakesrc");
+  fail_unless (src != NULL, "Could not create fakesrc");
+
+  sink = gst_element_factory_make ("fakesink", "fakesink");
+  fail_unless (sink != NULL, "Could not create fakesink");
+
+  g_object_set (sink, "state-error", 1, NULL);
+
+  gst_bin_add (GST_BIN (pipeline), src);
+  gst_bin_add (GST_BIN (pipeline), sink);
+
+  gst_element_link (src, sink);
+
+  bus = gst_pipeline_get_bus (GST_PIPELINE (pipeline));
+  fail_unless (bus != NULL, "Could not get bus");
+
+  gst_bus_set_sync_handler (bus, sync_handler_remove_sink, pipeline);
+
+  ret = gst_element_set_state (pipeline, GST_STATE_READY);
+  fail_unless (ret == GST_STATE_CHANGE_SUCCESS,
+      "did not get state change success");
+
+  gst_element_set_state (pipeline, GST_STATE_NULL);
+
+  gst_object_unref (bus);
+  gst_object_unref (pipeline);
+}
+
+GST_END_TEST;
+
 static Suite *
 gst_bin_suite (void)
 {
@@ -990,6 +1046,7 @@ gst_bin_suite (void)
   tcase_add_test (tc_chain, test_add_self);
   tcase_add_test (tc_chain, test_iterate_sorted);
   tcase_add_test (tc_chain, test_link_structure_change);
+  tcase_add_test (tc_chain, test_state_failure_remove);
 
   return s;
 }
